using System.ComponentModel;
using System.Xml.Serialization;

using Microsoft.Xna.Framework;

namespace EQIA.EQIDL
{
    /// <summary>
    /// Base class for any gui element that can be drawn on the screen.
    /// </summary>
    public partial class ScreenPiece : Class
    {
        #region Fields

        private Point myLocation;
        private Size mySize;
        private RGB myTextColor, myDisabledColor;

        private string myScreenID;
        private string myText;

        private int myFont;
        private int myTopAnchorOffset;
        private int myBottomAnchorOffset;
        private int myLeftAnchorOffset;
        private int myRightAnchorOffset;

        private bool isRelativePosition;
        private bool isAutoStretch;
        private bool isTopAnchorToTop;
        private bool isLeftAnchorToLeft;
        private bool isBottomAnchorToTop;
        private bool isRightAnchorToLeft;

        #endregion

        #region Properties

        #region Layout

        /// <summary>
        /// Gets or Sets the location of this screen piece from the top-left corner. Ignored if AutoStretch is true.
        /// </summary>
        [XmlElement("Location", IsNullable = false)]
        [Browsable(true), Description("(x, y) coordinates of top-left corner. Ignored if RelativePosition AND Autostretch is true.")]
        public Point Location { get { return myLocation; } set { myLocation = value; } }

        /// <summary>
        /// Gets or Sets the size of this screen piece. Ignored if AutoStretch is true.
        /// </summary>
        [XmlElement("Size", IsNullable = false)]
        [Browsable(true), Description("(w, h) of item. Ignored if RelativePosition AND Autostretch is true.")]
        public Size Size { get { return mySize; } set { mySize = value; } }

        /// <summary>
        /// Gets or Sets a value indicating if the location of this screen piece will be treated as a relative co-ordinate inside it's parent.
        /// </summary>
        [XmlElement("RelativePosition", IsNullable = false)]
        [Browsable(true), Description("Draw this @ (x, y) relative from its parent window's top-left corner")]
        public bool RelativePosition { get { return isRelativePosition; } set { isRelativePosition = value; } }

        /// <summary>
        /// Gets or Sets a value indicating if this screen piece should be sized and positioned according to it's anchors.
        /// </summary>
        [XmlElement("AutoStretch", IsNullable = false)]
        [Browsable(true), Description("Stretch this window to the borders of its parent. If true, this window will be resized when his parent is resized. If not, all anchor variables are ignored.")]
        public bool AutoStretch
        {
            get { return isAutoStretch; }
            set
            {
                isAutoStretch = value;
            }
        }

        /// <summary>
        /// Gets or Sets the anchor distance for the top edge of this screen piece.
        /// </summary>
        [XmlElement("TopAnchorOffset", IsNullable = false)]
        [Browsable(true), Description("Used by TopAnchorToTop")]
        public int TopAnchorOffset 
        { 
            get { return myTopAnchorOffset; } 
            set 
            {
                myTopAnchorOffset = value; 
            } 
        }

        /// <summary>
        /// Gets or Sets the anchor distance for the bottom edge of this screen piece.
        /// </summary>
        [XmlElement("BottomAnchorOffset", IsNullable = false)]
        [Browsable(true), Description("Used by BottomAnchorToTop")]
        public int BottomAnchorOffset 
        { 
            get { return myBottomAnchorOffset; } 
            set 
            { 
                myBottomAnchorOffset = value; 
            } 
        }

        /// <summary>
        /// Gets or Sets the anchor distance for the left edge of this screen piece.
        /// </summary>
        [XmlElement("LeftAnchorOffset", IsNullable = false)]
        [Browsable(true), Description("Used by LeftAnchorToLeft")]
        public int LeftAnchorOffset 
        { 
            get { return myLeftAnchorOffset; }
            set 
            { 
                myLeftAnchorOffset = value; 
            } 
        }

        /// <summary>
        /// Gets or Sets the anchor distance for the right edge of this screen piece.
        /// </summary>
        [XmlElement("RightAnchorOffset", IsNullable = false)]
        [Browsable(true), Description("Used by RightAnchorToLeft")]
        public int RightAnchorOffset 
        { 
            get { return myRightAnchorOffset; }
            set
            {
                myRightAnchorOffset = value; 
            }
        }

        /// <summary>
        /// Gets or Sets a value indicating if the top anchor offset for this screen piece should be from the top (true) or the bottom (false).
        /// </summary>
        [XmlElement("TopAnchorToTop", IsNullable = false)]
        [Browsable(true), Description("If true, keep the top side of this window a fixed offset away from its parent's top. Else, keep it a fixed offset away from its parent's bottom.")]
        public bool TopAnchorToTop 
        { 
            get { return isTopAnchorToTop; } 
            set 
            {
                isTopAnchorToTop = value; 
            }
        }

        /// <summary>
        /// Gets or Sets a value indicating if the left anchor offset for this screen piece should be from the left (true) or the right (false).
        /// </summary>
        [XmlElement("LeftAnchorToLeft", IsNullable = false)]
        [Browsable(true), Description("If true, keep the left side of this window a fixed offset away from its parent's left. Else, keep it a fixed offset away from its parent's right.")]
        public bool LeftAnchorToLeft
        { 
            get { return isLeftAnchorToLeft; }
            set 
            {
                isLeftAnchorToLeft = value;
            } 
        }

        /// <summary>
        /// Gets or Sets a value indicating if the bottom anchor offset for this screen piece should be from the top (true) or the bottom (false).
        /// </summary>
        [XmlElement("BottomAnchorToTop", IsNullable = false)]
        [Browsable(true), Description("If true, keep the bottom side of this window a fixed offset away from its parent's top. Else, keep it a fixed offset away from its parent's bottom.")]
        public bool BottomAnchorToTop { get { return isBottomAnchorToTop; } set { isBottomAnchorToTop = value; } }

        /// <summary>
        /// Gets or Sets a value indicating if the right anchor offset for this screen piece should be from the left (true) or the right (false).
        /// </summary>
        [XmlElement("RightAnchorToLeft", IsNullable = false)]
        [Browsable(true), Description("If true, keep the right side of this window a fixed offset away from its parent's left. Else, keep it a fixed offset away from its parent's right.")]
        public bool RightAnchorToLeft { get { return isRightAnchorToLeft; } set { isRightAnchorToLeft = value; } }

        #endregion

        /// <summary>
        /// Gets or Sets the color used for text in this screen piece.
        /// </summary>
        [XmlElement("TextColor", IsNullable = false)]
        [Browsable(true), Description("Color of the main text")]
        public RGB TextColor { get { return myTextColor; } set { myTextColor = value; } }

        /// <summary>
        /// Gets or Sets the color used for text in this screen piece when it's disabled.
        /// </summary>
        [XmlElement("DisabledColor", IsNullable = false)]
        [Browsable(true), Description("Disabled color of the main text")]
        public RGB DisabledColor { get { return myDisabledColor; } set { myDisabledColor = value; } }

        /// <summary>
        /// Gets or Sets an identifier which is unique to all elements in this screen piece's parent.
        /// </summary>
        [XmlElement("ScreenID", IsNullable = false)]
        [Browsable(true), Description("An identifier that is unique on the scope of all items being created within a parent control. All parent controls can access any of their children by this ID.")]
        public string ScreenID { get { return myScreenID; } set { myScreenID = value; } }

        /// <summary>
        /// Gets or Sets the text displayed in this screen piece.
        /// </summary>
        [XmlElement("Text", IsNullable = false)]
        [Browsable(true), Description("Main text for this item.")]
        public string Text { get { return myText; } set { myText = value; } }

        /// <summary>
        /// Gets or Sets the size of the font in this screen piece (1 to 5 inclusively).
        /// </summary>
        [XmlElement("Font", IsNullable = false)]
        [Browsable(true), Description("Font style from 0 - 6 (0 being small, 6 being large)")]
        public int Font { get { return myFont; } set { myFont = value; } }

        #endregion

        #region Creation

        public ScreenPiece()
            : this(null)
        {
        }

        public ScreenPiece(EQInterface eqInterface)
            : base(eqInterface)
        {
        }

        #endregion

        #region Deserialization

        protected override void SetProperties(object obj)
        {
            base.SetProperties(obj);

            ScreenPiece temp_ScreenPiece = obj as ScreenPiece;
            if (temp_ScreenPiece != null)
            {
                ScreenID = temp_ScreenPiece.ScreenID;
                Text = temp_ScreenPiece.Text;
                Font = temp_ScreenPiece.Font;
                TopAnchorOffset = temp_ScreenPiece.TopAnchorOffset;
                BottomAnchorOffset = temp_ScreenPiece.BottomAnchorOffset;
                LeftAnchorOffset = temp_ScreenPiece.LeftAnchorOffset;
                RightAnchorOffset = temp_ScreenPiece.RightAnchorOffset;
                RelativePosition = temp_ScreenPiece.RelativePosition;
                AutoStretch = temp_ScreenPiece.AutoStretch;
                TopAnchorToTop = temp_ScreenPiece.TopAnchorToTop;
                LeftAnchorToLeft = temp_ScreenPiece.LeftAnchorToLeft;
                BottomAnchorToTop = temp_ScreenPiece.BottomAnchorToTop;
                RightAnchorToLeft = temp_ScreenPiece.RightAnchorToLeft;
                Location = temp_ScreenPiece.Location;
                Size = temp_ScreenPiece.Size;
                TextColor = temp_ScreenPiece.TextColor;
                DisabledColor = temp_ScreenPiece.DisabledColor;
            }
        }

        #endregion
    }
}
